//*********************************************************//
// Name:      EtherCard + DS18B20 Test                     //
// Author:    Ilya Danilov www.mk90.ru/blog                //
// Date:      07 May 2013, LAST UPDATE: 18 May 2013        //
// Version:   0.0.1                                        //
// Schematic: Ардуино-совместимая плата на ATmega328P +    //
//            EtherCard (SS=D10) + DS18B20 (D9)            //
// Note:      При обращении через браузер выводятся        //
//            ROM-идентификаторы датчиков, обнаруженные    //
//            на шине. См. также mk90.blogspot.com/DS18B20 //    
// Based on EtherCard project: github.com/jcw/ethercard    //
//*********************************************************//

#include <OneWire.h>
#include <EtherCard.h>

// настройки Ethernet

#define BUF_SIZE 512

byte mac[] = { 0x00, 0x04, 0xA3, 0x21, 0xCA, 0x38 }; // MAC-адрес

byte fixed = false; // =false: пробовать получить адрес по DHCP, 
                    //         в случае неудачи использовать статический; 
                    // =true:  сразу использовать статический

uint8_t ip[] = { 169, 254, 8, 200 };     // Статический IP-адрес
uint8_t subnet[] = { 255, 255, 0, 0 };   // Маска подсети
uint8_t gateway[] = { 192, 168, 1, 20 }; // Адрес шлюза 
uint8_t dns[] = { 192, 168, 1, 20 };     // Адрес DNS-сервера (необязателен)

byte Ethernet::buffer[BUF_SIZE];
static BufferFiller bfill; 

// настройки OneWire

#define DS18B20PIN  9
OneWire ds(DS18B20PIN);

void setup(void)
{
    Serial.begin(57600);
    delay(2000);

    /* Проверяем, что контроллер Ethernet доступен для работы */
    Serial.println("Initialising the Ethernet controller");
    if (ether.begin(sizeof Ethernet::buffer, mac, 10) == 0) {
        Serial.println( "Ethernet controller NOT initialised");
        while (true);
    }

    // Пытаемся получить адрес динамически 
    Serial.println("Attempting to get an IP address using DHCP");
    if (ether.dhcpSetup()) {
        ether.printIp("Got an IP address using DHCP: ", ether.myip);
    } else {
        // Если DHCP не доступен, используем статический ip-адрес
        ether.staticSetup(ip, gateway, dns);
        ether.printIp("DHCP FAILED, using fixed address: ", ether.myip);
        fixed = true;
    }
}

char okHeader[] PROGMEM =
  "HTTP/1.0 200 OK\r\n"
  "Content-Type: text/html\r\n"
  "Pragma: no-cache\r\n"
;

char authorLink[] PROGMEM =
  "</pre><hr>Read about me <a href=\"http://mk90.blogspot.com\" target=\"_blank\">here</a>"
;

static void homePage (BufferFiller& buf) {
  
  buf.emit_p(PSTR("$F\r\n"
    "<title>Arduino web-thermometr</title>"
    "<h2>DS18B20 Network:</h2>"
    "<pre>"), okHeader);
  
  byte counter = 0; 
  byte addr[8]; 
  ds.reset_search();
  delay(250);
  while (ds.search(addr)) {
    buf.emit_p(PSTR("$D: "),++counter);
    for (byte i=0; i<8; i++) {  // считываем и выводим 9-байтный код
      buf.emit_p(PSTR("$H "), addr[i]);
    }
    if ( OneWire::crc8(addr, 7) != addr[7]) {
      buf.emit_p(PSTR("- CRC is not valid!"));
    } else if (addr[0] != 0x28) {
      buf.emit_p(PSTR("- is not a DS18B20 family device!"));
    }
    buf.emit_p(PSTR("\n"));
  }
  buf.emit_p(PSTR("\nTotal: $D devices."), counter);
  buf.emit_p(authorLink);
}


void loop(void)
{
  word len = ether.packetReceive();
  word pos = ether.packetLoop(len);
  if (pos) { 
    bfill = ether.tcpOffset();
    char* data = (char *) Ethernet::buffer + pos;
    Serial.println(data); // распечатываем запрос для отладки
    
    if (strncmp("GET / ", data, 6) == 0) 
      homePage(bfill);
    else
      bfill.emit_p(PSTR(
        "HTTP/1.0 401 Unauthorized\r\n"
        "Content-Type: text/html\r\n"
        "\r\n"
        "<h1>401 Unauthorized</h1>"));
        
    // отправить ответ клиенту
    ether.httpServerReply(bfill.position());
  }
}
